DTYPES = {
    "smallint": "smallint",
    "bigint": "bigint",
    "int": "int",
    "integer": "integer",
    "tinyint": "tinyint",
    "byteint": "byteint",
    "float": "float",
    "float4": "float4",
    "float8": "float8",
    "float64": "float64",
    "decimal": "decimal",
    "numeric": "numeric",
    "number": "number",
    "real": "real",
    "double": "double",
    "string": "string",
    "text": "text",
    "varchar": "varchar",
    "char": "char",
    "character": "character",
    "date": "date",
    "datetime": "datetime",
    "time": "time",
    "timestamp": "timestamp",
    "timestamp_ltz": "timestamp_ltz",
    "timestamp_ntz": "timestamp_ntz",
    "timestamp_tz": "timestamp_tz",
    "binary": "binary",
    "varbinary": "varbinary",
    "boolean": "boolean",
    "bool": "bool",
    "variant": "variant",
    "object": "object",
    "array": "array",
    # Aliases
    "double precision": "double",
    "doubleprecision": "double",
}
